/*
 * $Id:ConfigElement.java 456 2008-01-05 21:56:57Z andreamedeghini $
 *
 * JAME is a Java real-time multi-thread fractal graphics platform
 * Copyright (C) 2001, 2008 Andrea Medeghini
 * andreamedeghini@users.sf.net
 * http://jame.sourceforge.net
 * http://sourceforge.net/projects/jame
 * http://jame.dev.java.net
 * http://jugbrescia.dev.java.net
 *
 * This file is part of JAME.
 *
 * JAME is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JAME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JAME.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sf.jame.core.config;

import java.io.Serializable;

/**
 * Configuration element.
 * 
 * @author Andrea Medeghini
 */
public interface ConfigElement extends Serializable, Cloneable {
	/**
	 * Sets the configuration context.
	 * 
	 * @param context the configuration context to set.
	 */
	public void setContext(ConfigContext context);

	/**
	 * Returns the classId.
	 * 
	 * @return the classId.
	 */
	public String getClassId();

	/**
	 * Adds a listener.
	 * 
	 * @param listener the listener to add.
	 */
	public void addChangeListener(ValueChangeListener listener);

	/**
	 * Removes a listener.
	 * 
	 * @param listener the listener to remove.
	 */
	public void removeChangeListener(ValueChangeListener listener);

	/**
	 * @return
	 */
	public ConfigElement clone();

	/**
	 * 
	 */
	public void dispose();

	/**
	 * @return
	 */
	public Object getUserData();

	/**
	 * @param data
	 */
	public void setUserData(Object userData);
}
