/*
 * $Id:ExtensionRegistry.java 456 2008-01-05 21:56:57Z andreamedeghini $
 *
 * JAME is a Java real-time multi-thread fractal graphics platform
 * Copyright (C) 2001, 2008 Andrea Medeghini
 * andreamedeghini@users.sf.net
 * http://jame.sourceforge.net
 * http://sourceforge.net/projects/jame
 * http://jame.dev.java.net
 * http://jugbrescia.dev.java.net
 *
 * This file is part of JAME.
 *
 * JAME is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JAME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JAME.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sf.jame.core.extension;

import java.util.List;

/**
 * Interface of extension registries.
 * 
 * @author Andrea Medeghini
 * @param <T> the extension runtime type.
 */
public interface ExtensionRegistry<T extends ExtensionRuntime> {
	/**
	 * Returns an extension.
	 * 
	 * @param extensionId the extensionId.
	 * @return the extension.
	 * @throws ExtensionNotFoundException if the extension can't be found.
	 */
	public Extension<T> getExtension(String extensionId) throws ExtensionNotFoundException;

	/**
	 * Returns the list of extensions.
	 * 
	 * @return the list of extensions.
	 */
	public List<Extension<T>> getExtensionList();
}
