/*
 * $Id:DefaultRenderedPalette.java 456 2008-01-05 21:56:57Z andreamedeghini $
 *
 * JAME is a Java real-time multi-thread fractal graphics platform
 * Copyright (C) 2001, 2008 Andrea Medeghini
 * andreamedeghini@users.sf.net
 * http://jame.sourceforge.net
 * http://sourceforge.net/projects/jame
 * http://jame.dev.java.net
 * http://jugbrescia.dev.java.net
 *
 * This file is part of JAME.
 *
 * JAME is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JAME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JAME.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sf.jame.mandelbrot.util;

import java.awt.Color;

/**
 * @author Andrea Medeghini
 */
public class DefaultRenderedPalette extends RenderedPalette {
	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */
	public DefaultRenderedPalette() {
		this(Color.WHITE, Color.BLACK);
	}

	/**
	 * @param firstColor
	 * @param lastColor
	 */
	public DefaultRenderedPalette(final Color firstColor, final Color lastColor) {
		super(DefaultRenderedPalette.buildParams(firstColor.getRGB(), lastColor.getRGB()));
	}

	/**
	 * @param firstColor
	 * @param lastColor
	 */
	public DefaultRenderedPalette(final int firstColor, final int lastColor) {
		super(DefaultRenderedPalette.buildParams(firstColor, lastColor));
	}

	/**
	 * @param firstColor
	 * @param lastColor
	 * @return the params.
	 * @throws Error
	 */
	private static RenderedPaletteParam[] buildParams(final int firstColor, final int lastColor) {
		final RenderedPaletteParam[] params = new RenderedPaletteParam[1];
		params[0] = new DefaultRenderedPaletteParam(new int[] { firstColor, lastColor }, 100);
		return params;
	}
}
