/*
 * $Id:SimpleRenderedImage.java 456 2008-01-05 21:56:57Z andreamedeghini $
 *
 * JAME is a Java real-time multi-thread fractal graphics platform
 * Copyright (C) 2001, 2008 Andrea Medeghini
 * andreamedeghini@users.sf.net
 * http://jame.sourceforge.net
 * http://sourceforge.net/projects/jame
 * http://jame.dev.java.net
 * http://jugbrescia.dev.java.net
 *
 * This file is part of JAME.
 *
 * JAME is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JAME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JAME.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sf.jame.media;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;

public final class SimpleRenderedImage extends AbstractRenderedImage {
	private Controller controller;
	private Movie parent;
	private Layer layer;
	private Sequence sequence;
	private final EngineRenderedImage image;
	private final String name;
	private final Point2D center;
	private final AffineTransform transform;

	public SimpleRenderedImage(final String name, final BufferedImage image, final Renderer renderer) {
		this(name, new EngineRenderedImage(image, renderer));
	}

	private SimpleRenderedImage(final String name, final EngineRenderedImage image) {
		this.image = image;
		this.name = name;
		center = new Point2D.Float(image.getWidth() / 2, image.getHeight() / 2);
		transform = AffineTransform.getTranslateInstance(-center.getX(), -center.getY());
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		EngineRenderedImage img = null;
		if (image != null) {
			img = (EngineRenderedImage) image.clone();
		}
		final SimpleRenderedImage copy = new SimpleRenderedImage(name + "_copy", img);
		copy.setTransform((AffineTransform) transform.clone());
		return copy;
	}

	protected Layer getLayer() {
		return layer;
	}

	protected Sequence getSequence() {
		return sequence;
	}

	protected Controller getController() {
		return controller;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public Movie getParent() {
		return parent;
	}

	@Override
	public Point2D getCenter() {
		return center;
	}

	@Override
	public AffineTransform getTransform() {
		return transform;
	}

	@Override
	public BufferedImage getImage() {
		return image.getImage();
	}

	@Override
	public Renderer getRenderer() {
		return image.getRenderer();
	}

	@Override
	void build(final Controller controller, final Movie parent, final Layer layer, final Sequence sequence) {
		this.controller = controller;
		this.parent = parent;
		this.layer = layer;
		this.sequence = sequence;
		if ((image != null) && (image.renderer != null)) {
			image.renderer.build(controller, parent, layer, sequence);
		}
	}

	@Override
	void init() {
		if ((image != null) && (image.renderer != null)) {
			image.renderer.init();
		}
	}

	@Override
	void kill() {
		if ((image != null) && (image.renderer != null)) {
			image.renderer.kill();
		}
	}

	@Override
	void reset() {
		if ((image != null) && (image.renderer != null)) {
			image.renderer.reset();
		}
	}

	@Override
	void applyEffect(final Effect effect) {
		if (image != null) {
			image.applyEffect(effect);
		}
	}

	public void nextFrame() {
		image.getRenderer().nextFrame();
	}

	public void prevFrame() {
		image.getRenderer().prevFrame();
	}
}
