/*
 * $Id:TestMovieClipFileCache.java 456 2008-01-05 21:56:57Z andreamedeghini $
 *
 * JAME is a Java real-time multi-thread fractal graphics platform
 * Copyright (C) 2001, 2008 Andrea Medeghini
 * andreamedeghini@users.sf.net
 * http://jame.sourceforge.net
 * http://sourceforge.net/projects/jame
 * http://jame.dev.java.net
 * http://jugbrescia.dev.java.net
 *
 * This file is part of JAME.
 *
 * JAME is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JAME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JAME.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sf.jame.service.test;

import java.io.File;

import net.sf.jame.service.ConnectionFactory;
import net.sf.jame.service.DefaultConnectionFactory;
import net.sf.jame.service.Session;
import net.sf.jame.service.clip.MovieClip;
import net.sf.jame.service.clip.MovieClipDataRow;
import net.sf.jame.service.clip.MovieClipFileCache;

import org.junit.Assert;
import org.junit.Test;

public class TestMovieClipFileCache {
	private int lastId = 1;

	@Test
	public void testCreate() {
		try {
			final MovieClipFileCache fileCache = new MovieClipFileCache(new File("workdir/clip"));
			final ConnectionFactory factory = new DefaultConnectionFactory(new File("workdir"));
			final Session session = new Session(factory);
			session.openTransaction();
			fileCache.deleteAll();
			fileCache.create(createClip().getClipId());
			fileCache.create(createClip().getClipId());
			fileCache.create(createClip().getClipId());
			fileCache.create(createClip().getClipId());
			final File[] clips = fileCache.list();
			printClips(clips);
			Assert.assertEquals(4, clips.length);
			session.closeTransaction();
			session.close();
		}
		catch (final Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	}

	@Test
	public void testDelete() {
		try {
			final MovieClipFileCache fileCache = new MovieClipFileCache(new File("workdir/clip"));
			final ConnectionFactory factory = new DefaultConnectionFactory(new File("workdir"));
			final Session session = new Session(factory);
			session.openTransaction();
			fileCache.deleteAll();
			final MovieClipDataRow clip = createClip();
			fileCache.create(clip.getClipId());
			fileCache.delete(clip.getClipId());
			final File[] clips = fileCache.list();
			Assert.assertEquals(0, clips.length);
			session.closeTransaction();
			session.close();
		}
		catch (final Exception e) {
			e.printStackTrace();
			Assert.fail();
		}
	}

	private MovieClipDataRow createClip() {
		final MovieClipDataRow clip = new MovieClipDataRow(new MovieClip());
		clip.setClipId(lastId);
		clip.setClipName("Test Name");
		clip.setDescription("Test Description");
		lastId += 1;
		return clip;
	}

	private void printClips(final File[] clips) {
		for (final File clip : clips) {
			System.out.println(clip);
		}
	}
}
