/*
 * $Id:BlurRuntime.java 456 2008-01-05 21:56:57Z andreamedeghini $
 *
 * JAME is a Java real-time multi-thread fractal graphics platform
 * Copyright (C) 2001, 2008 Andrea Medeghini
 * andreamedeghini@users.sf.net
 * http://jame.sourceforge.net
 * http://sourceforge.net/projects/jame
 * http://jame.dev.java.net
 * http://jugbrescia.dev.java.net
 *
 * This file is part of JAME.
 *
 * JAME is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JAME is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JAME.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package net.sf.jame.twister.extensions.frame.layer.filter;

import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;

import net.sf.jame.twister.ImageTile;
import net.sf.jame.twister.frame.layer.filter.extension.LayerFilterExtensionRuntime;
import net.sf.jame.twister.renderer.Padding;
import net.sf.jame.twister.renderer.Surface;

/**
 * @author Andrea Medeghini
 */
public class BlurRuntime extends LayerFilterExtensionRuntime<BlurConfig> {
	private final float[] kernel = { 0.1f, 0.1f, 0.1f, 0.1f, 0.20f, 0.1f, 0.1f, 0.1f, 0.1f };
	private final ConvolveOp convolveop = new ConvolveOp(new Kernel(3, 3, kernel), ConvolveOp.EDGE_NO_OP, null);
	private final Padding padding = new Padding(0, 0, 0, 0);

	/**
	 * @see net.sf.jame.twister.frame.layer.filter.extension.LayerFilterExtensionRuntime#getPadding()
	 */
	@Override
	public Padding getPadding() {
		return padding;
	}

	/**
	 * @see net.sf.jame.twister.frame.layer.filter.extension.LayerFilterExtensionRuntime#renderImage(net.sf.jame.twister.renderer.Surface, net.sf.jame.twister.renderer.Surface)
	 */
	@Override
	public void renderImage(final Surface src, final Surface dst) {
		convolveop.filter(src.getImage(), dst.getImage());
	}

	/**
	 * @see net.sf.jame.twister.frame.layer.filter.extension.LayerFilterExtensionRuntime#setTile(net.sf.jame.twister.ImageTile)
	 */
	@Override
	public void setTile(final ImageTile tile) {
	}

	/**
	 * @see net.sf.jame.twister.frame.layer.filter.extension.LayerFilterExtensionRuntime#prepareFilter()
	 */
	@Override
	public void prepareFilter() {
	}
}
